from .. import home

MALAY_TEXT = home + '/dictionary/malay-text.txt'

PATH_STEM = {
    'lstm': {
        'model': home + '/stem/lstm/lstm-stem.pb',
        'setting': home + '/stem/lstm/lstm-stem.json',
        'version': 'v15',
    },
    'bahdanau': {
        'model': home + '/stem/bahdanau/bahdanau-stem.pb',
        'setting': home + '/stem/bahdanau/bahdanau-stem.json',
        'version': 'v15',
    },
    'luong': {
        'model': home + '/stem/luong/luong-stem.pb',
        'setting': home + '/stem/luong/luong-stem.json',
        'version': 'v15',
    },
}

S3_PATH_STEM = {
    'lstm': {
        'model': 'v15/stem/lstm-stem.pb',
        'setting': 'v15/stem/lstm-stem.json',
    },
    'bahdanau': {
        'model': 'v15/stem/bahdanau-stem.pb',
        'setting': 'v15/stem/bahdanau-stem.json',
    },
    'luong': {
        'model': 'v15/stem/luong-stem.pb',
        'setting': 'v15/stem/luong-stem.json',
    },
}

PATH_SUMMARIZE = {
    'news': {
        'model': home + '/summarize/summary-news.pb',
        'setting': home + '/summarize/summary-news.json',
        'version': 'v13',
    },
    'wiki': {
        'model': home + '/summarize/summary-wiki.pb',
        'setting': home + '/summarize/summary-wiki.json',
        'version': 'v13',
    },
}

S3_PATH_SUMMARIZE = {
    'news': {
        'model': 'v13/summarize/summary-news.pb',
        'setting': 'v13/summarize/summary-news.json',
    },
    'wiki': {
        'model': 'v13/summarize/summary-wiki.pb',
        'setting': 'v13/summarize/summary-wiki.json',
    },
}

PATH_TOXIC = {
    'multinomial': {
        'model': home + '/toxic/multinomial/multinomials-toxic.pkl',
        'vector': home + '/toxic/multinomial/vectorizer-toxic.pkl',
        'version': 'v6',
    },
    'logistic': {
        'model': home + '/toxic/logistic/logistics-toxic.pkl',
        'vector': home + '/toxic/logistic/vectorizer-toxic.pkl',
        'version': 'v6',
    },
    'luong': {
        'model': home + '/toxic/luong/luong-toxic.pb',
        'setting': home + '/toxic/luong/luong-toxic.json',
        'version': 'v8',
    },
    'bahdanau': {
        'model': home + '/toxic/bahdanau/bahdanau-toxic.pb',
        'setting': home + '/toxic/bahdanau/bahdanau-toxic.json',
        'version': 'v8',
    },
    'hierarchical': {
        'model': home + '/toxic/hierarchical/hierarchical-toxic.pb',
        'setting': home + '/toxic/hierarchical/hierarchical-toxic.json',
        'version': 'v8',
    },
    'fast-text': {
        'model': home + '/toxic/fast-text/fasttext-toxic.pb',
        'setting': home + '/toxic/fast-text/fasttext-toxic.json',
        'pickle': home + '/toxic/fast-text/fasttext-toxic.pkl',
        'version': 'v8',
    },
    'entity-network': {
        'model': home + '/toxic/entity-network/entity-toxic.pb',
        'setting': home + '/toxic/entity-network/entity-toxic.json',
        'version': 'v8',
    },
}

S3_PATH_TOXIC = {
    'multinomial': {
        'model': 'v6/multinomials-toxic.pkl',
        'vector': 'v6/vectorizer-toxic.pkl',
    },
    'logistic': {
        'model': 'v6/logistics-toxic.pkl',
        'vector': 'v6/vectorizer-toxic.pkl',
    },
    'luong': {
        'model': 'v8/toxic/luong-toxic.pb',
        'setting': 'v8/toxic/luong-toxic.json',
    },
    'bahdanau': {
        'model': 'v8/toxic/bahdanau-toxic.pb',
        'setting': 'v8/toxic/bahdanau-toxic.json',
    },
    'hierarchical': {
        'model': 'v8/toxic/hierarchical-toxic.pb',
        'setting': 'v8/toxic/hierarchical-toxic.json',
    },
    'fast-text': {
        'model': 'v8/toxic/fasttext-toxic.pb',
        'setting': 'v8/toxic/fasttext-toxic.json',
        'pickle': 'v8/toxic/fasttext-toxic.pkl',
    },
    'entity-network': {
        'model': 'v8/toxic/entity-toxic.pb',
        'setting': 'v8/toxic/entity-toxic.json',
    },
}

PATH_POS = {
    'crf': {'model': home + '/pos/crf/crf-pos.pkl', 'version': 'v14'},
    'concat': {
        'model': home + '/pos/concat/concat-pos.pb',
        'setting': home + '/pos/concat/concat-pos.json',
        'version': 'v14',
    },
    'luong': {
        'model': home + '/pos/luong/luong-pos.pb',
        'setting': home + '/pos/luong/luong-pos.json',
        'version': 'v14',
    },
    'bahdanau': {
        'model': home + '/pos/bahdanau/bahdanau-pos.pb',
        'setting': home + '/pos/bahdanau/bahdanau-pos.json',
        'version': 'v14',
    },
    'entity-network': {
        'model': home + '/pos/entity-network/entity-pos.pb',
        'setting': home + 'pos/entity-network/entity-pos.json',
        'version': 'v14',
    },
    'attention': {
        'model': home + '/pos/attention/attention-pos.pb',
        'setting': home + '/pos/attention/attention-pos.json',
        'version': 'v14',
    },
}

S3_PATH_POS = {
    'crf': {'model': 'v14/pos/crf-pos.pkl'},
    'concat': {
        'model': 'v14/pos/concat-pos.pb',
        'setting': 'v14/pos/concat-pos.json',
    },
    'luong': {
        'model': 'v14/pos/luong-pos.pb',
        'setting': 'v14/pos/luong-pos.json',
    },
    'bahdanau': {
        'model': 'v14/pos/bahdanau-pos.pb',
        'setting': 'v14/pos/bahdanau-pos.json',
    },
    'entity-network': {
        'model': 'v14/pos/entity-pos.pb',
        'setting': 'v14/pos/entity-pos.json',
    },
    'attention': {
        'model': 'v14/pos/attention-pos.pb',
        'setting': 'v14/pos/attention-pos.json',
    },
}

PATH_NORMALIZER = {
    'deep': {
        'model': home + '/normalizer/normalizer-deep.pb',
        'setting': home + '/normalizer/normalizer-deep.json',
        'version': 'v6',
    }
}

S3_PATH_NORMALIZER = {
    'deep': {
        'model': 'v6/normalizer-deep.pb',
        'setting': 'v6/normalizer-deep.json',
    }
}

PATH_LANG_DETECTION = {
    'multinomial': {
        'model': home
        + '/language-detection/multinomial/multinomial-language-detection.pkl',
        'vector': home
        + '/language-detection/multinomial/vectorizer-language-detection.pkl',
        'version': 'v10.2',
    },
    'xgb': {
        'model': home + '/language-detection/xgb/xgb-language-detection.pkl',
        'vector': home
        + '/language-detection/multinomial/vectorizer-language-detection.pkl',
        'version': 'v10.2',
    },
    'sgd': {
        'model': home + '/language-detection/sgd/sgd-language-detection.pkl',
        'vector': home
        + '/language-detection/multinomial/vectorizer-language-detection.pkl',
        'version': 'v10.2',
    },
    'deep': {
        'model': home
        + '/language-detection/deep/model.ckpt.data-00000-of-00001',
        'index': home + '/language-detection/deep/model.ckpt.index',
        'meta': home + '/language-detection/deep/model.ckpt.meta',
        'vector': home
        + '/language-detection/multinomial/vectorizer-language-detection.pkl',
        'version': 'v10.2',
    },
}

S3_PATH_LANG_DETECTION = {
    'multinomial': {
        'model': 'v10/language-detection/multinomial-language-detection.pkl',
        'vector': 'v10/language-detection/language-detection-vectorizer.pkl',
    },
    'xgb': {
        'model': 'v10/language-detection/xgboost-language-detection.pkl',
        'vector': 'v10/language-detection/language-detection-vectorizer.pkl',
    },
    'sgd': {
        'model': 'v10/language-detection/sgd-language-detection.pkl',
        'vector': 'v10/language-detection/language-detection-vectorizer.pkl',
    },
    'deep': {
        'model': 'v10/language-detection/model.ckpt.data-00000-of-00001',
        'index': 'v10/language-detection/model.ckpt.index',
        'meta': 'v10/language-detection/model.ckpt.meta',
        'vector': 'v10/language-detection/language-detection-vectorizer.pkl',
    },
}

PATH_ENTITIES = {
    'crf': {'model': home + '/entity/crf/crf-entities.pkl', 'version': 'v8'},
    'concat': {
        'model': home + '/entity/concat/concat-entities.pb',
        'setting': home + '/entity/concat/concat-entities.json',
        'version': 'v14',
    },
    'luong': {
        'model': home + '/entity/luong/luong-entities.pb',
        'setting': home + '/entity/luong/luong-entities.json',
        'version': 'v14',
    },
    'bahdanau': {
        'model': home + '/entity/bahdanau/bahdanau-entities.pb',
        'setting': home + '/entity/bahdanau/bahdanau-entities.json',
        'version': 'v14',
    },
    'entity-network': {
        'model': home + '/entity/entity-network/entity-entities.pb',
        'setting': home + '/entity/entity-network/entity-entities.json',
        'version': 'v14',
    },
    'attention': {
        'model': home + '/entity/attention/attention-entities.pb',
        'setting': home + '/entity/attention/attention-entities.json',
        'version': 'v14',
    },
}

S3_PATH_ENTITIES = {
    'crf': {'model': 'v8/entities/crf-entities.pkl'},
    'concat': {
        'model': 'v14/entities/concat-entities.pb',
        'setting': 'v14/entities/concat-entities.json',
    },
    'luong': {
        'model': 'v14/entities/luong-entities.pb',
        'setting': 'v14/entities/luong-entities.json',
    },
    'bahdanau': {
        'model': 'v14/entities/bahdanau-entities.pb',
        'setting': 'v14/entities/bahdanau-entities.json',
    },
    'entity-network': {
        'model': 'v14/entities/entity-entities.pb',
        'setting': 'v14/entities/entity-entities.json',
    },
    'attention': {
        'model': 'v14/entities/attention-entities.pb',
        'setting': 'v14/entities/attention-entities.json',
    },
}

PATH_SENTIMENTS = {
    'bert': {
        'model': home + '/sentiment/bert/bert-sentiment.pb',
        'setting': home + '/sentiment/bert/bert-sentiment.json',
        'version': 'v8',
    },
    'entity-network': {
        'model': home + '/sentiment/entity-network/entity-sentiment.pb',
        'setting': home + '/sentiment/entity-network/entity-sentiment.json',
        'version': 'v8',
    },
    'hierarchical': {
        'model': home + '/sentiment/hierarchical/hierarchical-sentiment.pb',
        'setting': home + '/sentiment/hierarchical/hierarchical-sentiment.json',
        'version': 'v8',
    },
    'bahdanau': {
        'model': home + '/sentiment/bahdanau/bahdanau-sentiment.pb',
        'setting': home + '/sentiment/bahdanau/bahdanau-sentiment.json',
        'version': 'v8',
    },
    'luong': {
        'model': home + '/sentiment/luong/luong-sentiment.pb',
        'setting': home + '/sentiment/luong/luong-sentiment.json',
        'version': 'v8',
    },
    'bidirectional': {
        'model': home + '/sentiment/bidirectional/bidirectional-sentiment.pb',
        'setting': home
        + '/sentiment/bidirectional/bidirectional-sentiment.json',
        'version': 'v8',
    },
    'fast-text': {
        'model': home + '/sentiment/fast-text/fasttext-sentiment.pb',
        'setting': home + '/sentiment/fast-text/fasttext-sentiment.json',
        'pickle': home + '/sentiment/fast-text/fasttext-sentiment.pkl',
        'version': 'v8',
    },
    'multinomial': {
        'model': home + '/sentiment/multinomial/multinomial-sentiment.pkl',
        'vector': home
        + '/sentiment/multinomial/multinomial-sentiment-tfidf.pkl',
        'version': 'v8',
    },
    'xgb': {
        'model': home + '/sentiment/xgb/xgboost-sentiment.pkl',
        'vector': home + '/sentiment/xgb/xgboost-sentiment-tfidf.pkl',
        'version': 'v8',
    },
    'fast-text-char': {
        'model': home
        + '/sentiment/fast-text-char/model.ckpt.data-00000-of-00001',
        'index': home + '/sentiment/fast-text-char/model.ckpt.index',
        'meta': home + '/sentiment/fast-text-char/model.ckpt.meta',
        'vector': home
        + '/sentiment/fast-text-char/vectorizer-sparse-sentiment.pkl',
        'version': 'v12',
    },
}

S3_PATH_SENTIMENTS = {
    'bert': {
        'model': 'v8/sentiment/bert-sentiment.pb',
        'setting': 'v8/sentiment/bert-sentiment.json',
    },
    'entity-network': {
        'model': 'v8/sentiment/entity-sentiment.pb',
        'setting': 'v8/sentiment/entity-sentiment.json',
    },
    'hierarchical': {
        'model': 'v8/sentiment/hierarchical-sentiment.pb',
        'setting': 'v8/sentiment/hierarchical-sentiment.json',
    },
    'bahdanau': {
        'model': 'v8/sentiment/bahdanau-sentiment.pb',
        'setting': 'v8/sentiment/bahdanau-sentiment.json',
    },
    'luong': {
        'model': 'v8/sentiment/luong-sentiment.pb',
        'setting': 'v8/sentiment/luong-sentiment.json',
    },
    'bidirectional': {
        'model': 'v8/sentiment/bidirectional-sentiment.pb',
        'setting': 'v8/sentiment/bidirectional-sentiment.json',
    },
    'fast-text': {
        'model': 'v8/sentiment/fasttext-sentiment.pb',
        'setting': 'v8/sentiment/fasttext-sentiment.json',
        'pickle': 'v8/sentiment/fasttext-sentiment.pkl',
    },
    'multinomial': {
        'model': 'v8/sentiment/multinomial-sentiment.pkl',
        'vector': 'v8/sentiment/multinomial-sentiment-tfidf.pkl',
    },
    'xgb': {
        'model': 'v8/sentiment/xgboost-sentiment.pkl',
        'vector': 'v8/sentiment/xgboost-sentiment-tfidf.pkl',
    },
    'fast-text-char': {
        'model': 'v12/sentiment/model.ckpt.data-00000-of-00001',
        'index': 'v12/sentiment/model.ckpt.index',
        'meta': 'v12/sentiment/model.ckpt.meta',
        'vector': 'v12/sentiment/vectorizer-sparse-sentiment.pkl',
    },
}

PATH_SUBJECTIVE = {
    'bert': {
        'model': home + '/subjective/bert/bert-subjective.pb',
        'setting': home + '/subjective/bert/bert-subjective.json',
        'version': 'v10',
    },
    'entity-network': {
        'model': home + '/subjective/entity-network/entity-subjective.pb',
        'setting': home + '/subjective/entity-network/entity-subjective.json',
        'version': 'v10',
    },
    'hierarchical': {
        'model': home + '/subjective/hierarchical/hierarchical-subjective.pb',
        'setting': home
        + '/subjective/hierarchical/hierarchical-subjective.json',
        'version': 'v10',
    },
    'bahdanau': {
        'model': home + '/subjective/bahdanau/bahdanau-subjective.pb',
        'setting': home + '/subjective/bahdanau/bahdanau-subjective.json',
        'version': 'v10',
    },
    'luong': {
        'model': home + '/subjective/luong/luong-subjective.pb',
        'setting': home + '/subjective/luong/luong-subjective.json',
        'version': 'v10',
    },
    'bidirectional': {
        'model': home + '/subjective/bidirectional/bidirectional-subjective.pb',
        'setting': home
        + '/subjective/bidirectional/bidirectional-subjective.json',
        'version': 'v10',
    },
    'fast-text': {
        'model': home + '/subjective/fast-text/fasttext-subjective.pb',
        'setting': home + '/subjective/fast-text/fasttext-subjective.json',
        'pickle': home + '/subjective/fast-text/fasttext-subjective.pkl',
        'version': 'v10',
    },
    'multinomial': {
        'model': home + '/subjective/multinomial/multinomial-subjective.pkl',
        'vector': home
        + '/subjective/multinomial/multinomial-subjective-tfidf.pkl',
        'version': 'v10',
    },
    'xgb': {
        'model': home + '/subjective/xgb/xgboost-subjective.pkl',
        'vector': home + '/subjective/xgb/xgboost-subjective-tfidf.pkl',
        'version': 'v10',
    },
    'fast-text-char': {
        'model': home
        + '/subjective/fast-text-char/model.ckpt.data-00000-of-00001',
        'index': home + '/subjective/fast-text-char/model.ckpt.index',
        'meta': home + '/subjective/fast-text-char/model.ckpt.meta',
        'vector': home
        + '/subjective/fast-text-char/vectorizer-sparse-subjective.pkl',
        'version': 'v12',
    },
}

S3_PATH_SUBJECTIVE = {
    'bert': {
        'model': 'v10/subjective/bert-subjective.pb',
        'setting': 'v10/subjective/bert-subjective.json',
    },
    'entity-network': {
        'model': 'v10/subjective/entity-subjective.pb',
        'setting': 'v10/subjective/entity-subjective.json',
    },
    'hierarchical': {
        'model': 'v10/subjective/hierarchical-subjective.pb',
        'setting': 'v10/subjective/hierarchical-subjective.json',
    },
    'bahdanau': {
        'model': 'v10/subjective/bahdanau-subjective.pb',
        'setting': 'v10/subjective/bahdanau-subjective.json',
    },
    'luong': {
        'model': 'v10/subjective/luong-subjective.pb',
        'setting': 'v10/subjective/luong-subjective.json',
    },
    'bidirectional': {
        'model': 'v10/subjective/bidirectional-subjective.pb',
        'setting': 'v10/subjective/bidirectional-subjective.json',
    },
    'fast-text': {
        'model': 'v10/subjective/fasttext-subjective.pb',
        'setting': 'v10/subjective/fasttext-subjective.json',
        'pickle': 'v10/subjective/fasttext-subjective.pkl',
    },
    'multinomial': {
        'model': 'v10/subjective/multinomial-subjective.pkl',
        'vector': 'v10/subjective/multinomial-subjective-tfidf.pkl',
    },
    'xgb': {
        'model': 'v10/subjective/xgboost-subjective.pkl',
        'vector': 'v10/subjective/xgboost-subjective-tfidf.pkl',
    },
    'fast-text-char': {
        'model': 'v12/subjective/model.ckpt.data-00000-of-00001',
        'index': 'v12/subjective/model.ckpt.index',
        'meta': 'v12/subjective/model.ckpt.meta',
        'vector': 'v12/subjective/vectorizer-sparse-subjective.pkl',
    },
}

PATH_EMOTION = {
    'bert': {
        'model': home + '/emotion/bert/bert-emotion.pb',
        'setting': home + '/emotion/bert/bert-emotion.json',
        'version': 'v12',
    },
    'entity-network': {
        'model': home + '/emotion/entity-network/entity-emotion.pb',
        'setting': home + '/emotion/entity-network/entity-emotion.json',
        'version': 'v12',
    },
    'hierarchical': {
        'model': home + '/emotion/hierarchical/hierarchical-emotion.pb',
        'setting': home + '/emotion/hierarchical/hierarchical-emotion.json',
        'version': 'v12',
    },
    'bahdanau': {
        'model': home + '/emotion/bahdanau/bahdanau-emotion.pb',
        'setting': home + '/emotion/bahdanau/bahdanau-emotion.json',
        'version': 'v12',
    },
    'luong': {
        'model': home + '/emotion/luong/luong-emotion.pb',
        'setting': home + '/emotion/luong/luong-emotion.json',
        'version': 'v12',
    },
    'bidirectional': {
        'model': home + '/emotion/bidirectional/bidirectional-emotion.pb',
        'setting': home + '/emotion/bidirectional/bidirectional-emotion.json',
        'version': 'v12',
    },
    'fast-text': {
        'model': home + '/emotion/fast-text/fasttext-emotion.pb',
        'setting': home + '/emotion/fast-text/fasttext-emotion.json',
        'pickle': home + '/emotion/fast-text/fasttext-emotion.pkl',
        'version': 'v12',
    },
    'multinomial': {
        'model': home + '/emotion/multinomial/multinomial-emotion.pkl',
        'vector': home + '/emotion/multinomial/multinomial-emotion-tfidf.pkl',
        'version': 'v12',
    },
    'xgb': {
        'model': home + '/emotion/xgb/xgboost-emotion.pkl',
        'vector': home + '/emotion/xgb/xgboost-emotion-tfidf.pkl',
        'version': 'v12',
    },
    'fast-text-char': {
        'model': home
        + '/emotion/fast-text-char/model.ckpt.data-00000-of-00001',
        'index': home + '/emotion/fast-text-char/model.ckpt.index',
        'meta': home + '/emotion/fast-text-char/model.ckpt.meta',
        'vector': home
        + '/emotion/fast-text-char/vectorizer-sparse-emotion.pkl',
        'version': 'v12',
    },
}

S3_PATH_EMOTION = {
    'bert': {
        'model': 'v12/emotion/bert-emotion.pb',
        'setting': 'v12/emotion/bert-emotion.json',
    },
    'entity-network': {
        'model': 'v12/emotion/entity-emotion.pb',
        'setting': 'v12/emotion/entity-emotion.json',
    },
    'hierarchical': {
        'model': 'v12/emotion/hierarchical-emotion.pb',
        'setting': 'v12/emotion/hierarchical-emotion.json',
    },
    'bahdanau': {
        'model': 'v12/emotion/bahdanau-emotion.pb',
        'setting': 'v12/emotion/bahdanau-emotion.json',
    },
    'luong': {
        'model': 'v12/emotion/luong-emotion.pb',
        'setting': 'v12/emotion/luong-emotion.json',
    },
    'bidirectional': {
        'model': 'v12/emotion/bidirectional-emotion.pb',
        'setting': 'v12/emotion/bidirectional-emotion.json',
    },
    'fast-text': {
        'model': 'v12/emotion/fasttext-emotion.pb',
        'setting': 'v12/emotion/fasttext-emotion.json',
        'pickle': 'v12/emotion/fasttext-emotion.pkl',
    },
    'multinomial': {
        'model': 'v12/emotion/multinomial-emotion.pkl',
        'vector': 'v12/emotion/multinomial-emotion-tfidf.pkl',
    },
    'xgb': {
        'model': 'v12/emotion/xgboost-emotion.pkl',
        'vector': 'v12/emotion/xgboost-emotion-tfidf.pkl',
    },
    'fast-text-char': {
        'model': 'v12/emotion/model.ckpt.data-00000-of-00001',
        'index': 'v12/emotion/model.ckpt.index',
        'meta': 'v12/emotion/model.ckpt.meta',
        'vector': 'v12/emotion/vectorizer-sparse-emotion.pkl',
    },
}

PATH_DEPEND = {
    'crf': {
        'model': home + '/dependency/crf/crf-label.pkl',
        'depend': home + '/dependency/crf/crf-depend.pkl',
        'version': 'v14',
    },
    'concat': {
        'model': home + '/dependency/concat/concat-dependency.pb',
        'setting': home + '/dependency/concat/concat-dependency.json',
        'version': 'v14',
    },
    'luong': {
        'model': home + '/dependency/luong/luong-dependency.pb',
        'setting': home + '/dependency/luong/luong-dependency.json',
        'version': 'v14',
    },
    'bahdanau': {
        'model': home + '/dependency/bahdanau/bahdanau-dependency.pb',
        'setting': home + '/dependency/bahdanau/bahdanau-dependency.json',
        'version': 'v14',
    },
}

S3_PATH_DEPEND = {
    'crf': {
        'model': 'v14/dependency/crf-label.pkl',
        'depend': 'v14/dependency/crf-depend.pkl',
    },
    'concat': {
        'model': 'v14/dependency/concat-dependency.pb',
        'setting': 'v14/dependency/concat-dependency.json',
    },
    'luong': {
        'model': 'v14/dependency/luong-dependency.pb',
        'setting': 'v14/dependency/luong-dependency.json',
    },
    'bahdanau': {
        'model': 'v14/dependency/bahdanau-dependency.pb',
        'setting': 'v14/dependency/bahdanau-dependency.json',
    },
}



import malaya

def test_pos_entities_char():
    available_models = malaya.get_available_pos_entities_models()
    string = 'KUALA LUMPUR: Sempena sambutan Aidilfitri minggu depan'
    assert len(malaya.deep_pos_entities('char').predict(string))

def test_pos_entities_concat():
    available_models = malaya.get_available_pos_entities_models()
    string = 'KUALA LUMPUR: Sempena sambutan Aidilfitri minggu depan'
    assert len(malaya.deep_pos_entities('concat').predict(string))

def test_pos_entities_attention():
    available_models = malaya.get_available_pos_entities_models()
    string = 'KUALA LUMPUR: Sempena sambutan Aidilfitri minggu depan'
    assert len(malaya.deep_pos_entities('attention').predict(string))

def test_pos_entities_attention_num():
    string = '34892347 23479 2312 35436 234'
    assert len(malaya.deep_pos_entities('attention').predict(string))



import sys
import warnings

if not sys.warnoptions:
    warnings.simplefilter('ignore')

import re
import os
import numpy as np
import itertools
import collections
from unidecode import unidecode
from .._utils._utils import download_file
from ._tatabahasa import stopword_tatabahasa
from .. import home

STOPWORDS = None
STOPWORD_CALON = [
    'datuk',
    'bin',
    'hj',
    'haji',
    'bn',
    'bnt',
    'prof',
    'binti',
    'dr',
    'ustaz',
    'mejar',
    'ir',
    'md',
    'tuan',
    'puan',
    'yb',
    'ustazah',
    'cikgu',
    'dato',
    'dsp',
]
stopwords_location = os.path.join(home, 'stop-word-kerulnet')
if not os.path.isfile(stopwords_location):
    print('downloading stopwords')
    download_file('stop-word-kerulnet', stopwords_location)
with open(stopwords_location, 'r') as fopen:
    STOPWORDS = list(filter(None, fopen.read().split('\n')))

STOPWORDS = set(STOPWORDS + stopword_tatabahasa + STOPWORD_CALON)
STOPWORD_CALON = set(STOPWORD_CALON)
VOWELS = 'aeiou'
PHONES = ['sh', 'ch', 'ph', 'sz', 'cz', 'sch', 'rz', 'dz']


def _isWord(word):
    if word:
        consecutiveVowels = 0
        consecutiveConsonents = 0
        for idx, letter in enumerate(word.lower()):
            vowel = True if letter in VOWELS else False
            if idx:
                prev = word[idx - 1]
                prevVowel = True if prev in VOWELS else False
                if not vowel and letter == 'y' and not prevVowel:
                    vowel = True
                if prevVowel != vowel:
                    consecutiveVowels = 0
                    consecutiveConsonents = 0
            if vowel:
                consecutiveVowels += 1
            else:
                consecutiveConsonents += 1
            if consecutiveVowels >= 3 or consecutiveConsonents > 3:
                return False
            if consecutiveConsonents == 3:
                subStr = word[idx - 2 : idx + 1]
                if any(phone in subStr for phone in PHONES):
                    consecutiveConsonents -= 1
                    continue
                return False
    return True


_list_laughing = {
    'huhu',
    'haha',
    'gaga',
    'hihi',
    'wkawka',
    'wkwk',
    'kiki',
    'keke',
    'huehue',
}


def malaya_textcleaning(string):
    """
    use by normalizer, spell
    remove links, hashtags, alias
    only accept A-Z, a-z
    remove any laugh
    remove any repeated char more than 2 times
    remove most of nonsense words
    """
    string = re.sub(
        'http\S+|www.\S+',
        '',
        ' '.join(
            [
                word
                for word in string.split()
                if word.find('#') < 0 and word.find('@') < 0
            ]
        ),
    )
    string = unidecode(string).replace('.', '. ')
    string = string.replace(',', ', ')
    string = re.sub('[^\'"A-Za-z\- ]+', ' ', string)
    string = re.sub(r'[ ]+', ' ', string.lower()).strip()
    string = [word for word in string.lower().split() if _isWord(word)]
    string = [
        word
        for word in string
        if not any([laugh in word for laugh in _list_laughing])
        and word[: len(word) // 2] != word[len(word) // 2 :]
    ]
    string = ' '.join(string)
    string = (
        ''.join(''.join(s)[:2] for _, s in itertools.groupby(string))
    ).split()
    return ' '.join([word for word in string if word not in STOPWORDS])


def normalizer_textcleaning(string):
    """
    use by normalizer, spell
    remove links, hashtags, alias
    only accept A-Z, a-z
    remove any laugh
    remove any repeated char more than 2 times
    """
    string = re.sub(
        'http\S+|www.\S+',
        '',
        ' '.join(
            [
                word
                for word in string.split()
                if word.find('#') < 0 and word.find('@') < 0
            ]
        ),
    )
    string = re.sub('[^A-Za-z ]+', ' ', string)
    string = re.sub(r'[ ]+', ' ', string).strip()
    string = [
        word.title() if word.isupper() else word
        for word in string.split()
        if len(word)
    ]
    string = [
        word
        for word in string
        if not any([laugh in word for laugh in _list_laughing])
    ]
    string = ' '.join(string)
    return ''.join(''.join(s)[:2] for _, s in itertools.groupby(string))


def simple_textcleaning(string, lowering = True):
    """
    use by topic modelling
    only accept A-Z, a-z
    """
    string = unidecode(string)
    string = re.sub('[^A-Za-z ]+', ' ', string)
    return re.sub(r'[ ]+', ' ', string.lower() if lowering else string).strip()


def entities_textcleaning(string):
    """
    use by xgb entities, multinomial entities,
    xgb pos, xgb entities, char model, word model, concat model
    """
    string = re.sub('[^A-Za-z0-9\-\/ ]+', ' ', string)
    string = re.sub(r'[ ]+', ' ', string).strip()
    return [
        word.title() if word.isupper() else word
        for word in string.split()
        if len(word)
    ]


def summary_textcleaning(string):
    string = re.sub('[^A-Za-z0-9\-\/\'"\.\, ]+', ' ', string)
    return re.sub(r'[ ]+', ' ', string.lower()).strip()


def get_hashtags(string):
    return [hash.lower() for hash in re.findall('#(\w+)', string)]


def split_by_dot(string):
    string = re.sub(
        r'(?<!\d)\.(?!\d)',
        'SPLITTT',
        string.replace('\n', '').replace('/', ' '),
    )
    string = string.split('SPLITTT')
    return [re.sub(r'[ ]+', ' ', sentence).strip() for sentence in string]


def language_detection_textcleaning(string):
    string = re.sub(
        u'[0-9!@#$%^&*()_\-+{}|\~`\'";:?/.>,<]',
        ' ',
        string.lower(),
        flags = re.UNICODE,
    )
    return re.sub(r'[ ]+', ' ', string).strip()


def pos_entities_textcleaning(string):
    """
    use by text entities and pos
    remove links, hashtags, alias
    """
    string = re.sub(
        'http\S+|www.\S+',
        '',
        ' '.join(
            [i for i in string.split() if i.find('#') < 0 and i.find('@') < 0]
        ),
    )
    string = unidecode(string).replace('.', '. ').replace(',', ', ')
    string = re.sub('[^A-Za-z\- ]+', ' ', string)
    string = re.sub(r'[ ]+', ' ', string).strip()
    return ' '.join(
        [
            word.title() if word.isupper() else word
            for word in string.split()
            if len(word)
        ]
    )


def classification_textcleaning(string, no_stopwords = False, lowering = True):
    """
    use by our text classifiers, stemmer, summarization, topic-modelling
    remove links, hashtags, alias
    """
    string = re.sub(
        'http\S+|www.\S+',
        '',
        ' '.join(
            [i for i in string.split() if i.find('#') < 0 and i.find('@') < 0]
        ),
    )
    string = unidecode(string).replace('.', '. ').replace(',', ', ')
    string = re.sub('[^A-Za-z ]+', ' ', string)
    string = re.sub(r'[ ]+', ' ', string).strip()
    if no_stopwords:
        string = ' '.join(
            [
                i
                for i in re.findall('[\\w\']+|[;:\-\(\)&.,!?"]', string)
                if len(i)
            ]
        )
    else:
        string = ' '.join(
            [
                i
                for i in re.findall('[\\w\']+|[;:\-\(\)&.,!?"]', string)
                if len(i) and i not in STOPWORDS
            ]
        )
    if lowering:
        return string.lower()
    else:
        return ' '.join(
            [
                word.title() if word.isupper() else word
                for word in string.split()
                if len(word)
            ]
        )


def process_word_pos_entities(word):
    word = word.lower()
    word = re.sub('[^A-Za-z0-9\- ]+', '', word)
    if word.isdigit():
        word = 'NUM'
    return word


def separate_dataset(trainset):
    datastring = []
    datatarget = []
    for i in range(len(trainset.data)):
        data_ = trainset.data[i].split('\n')
        data_ = list(filter(None, data_))
        datastring += data_
        for n in range(len(data_)):
            datatarget.append(trainset.target[i])
    return datastring, datatarget


def print_topics_modelling(
    topics, feature_names, sorting, topics_per_chunk = 6, n_words = 20
):
    for i in range(0, len(topics), topics_per_chunk):
        these_topics = topics[i : i + topics_per_chunk]
        len_this_chunk = len(these_topics)
        print(('topic {:<8}' * len_this_chunk).format(*these_topics))
        print(('-------- {0:<5}' * len_this_chunk).format(''))
        for i in range(n_words):
            print(
                ('{:<14}' * len_this_chunk).format(
                    *feature_names[sorting[these_topics, i]]
                )
            )
        print('\n')


def str_idx(corpus, dic, maxlen, UNK = 0):
    X = np.zeros((len(corpus), maxlen))
    for i in range(len(corpus)):
        for no, k in enumerate(corpus[i].split()[:maxlen][::-1]):
            X[i, -1 - no] = dic.get(k, UNK)
    return X


def stemmer_str_idx(corpus, dic, UNK = 3):
    X = []
    for i in corpus:
        ints = []
        for k in i:
            ints.append(dic.get(k, UNK))
        X.append(ints)
    return X


def fasttext_str_idx(texts, dictionary):
    idx_trainset = []
    for text in texts:
        idx = []
        for t in text.split():
            if t in dictionary:
                idx.append(dictionary[t])
        idx_trainset.append(idx)
    return idx_trainset


def pad_sentence_batch(sentence_batch, pad_int):
    padded_seqs = []
    seq_lens = []
    max_sentence_len = max([len(sentence) for sentence in sentence_batch])
    for sentence in sentence_batch:
        padded_seqs.append(
            sentence + [pad_int] * (max_sentence_len - len(sentence))
        )
        seq_lens.append(len(sentence))
    return padded_seqs, seq_lens


def add_ngram(sequences, token_indice, ngram = (2, 3)):
    new_sequences = []
    for input_list in sequences:
        new_list = input_list[:]
        for ngram_value in range(ngram[0], ngram[1]):
            for i in range(len(new_list) - ngram_value + 1):
                ngram = tuple(new_list[i : i + ngram_value])
                if ngram in token_indice:
                    new_list.append(token_indice[ngram])
        new_sequences.append(new_list)
    return new_sequences


def char_str_idx(corpus, dic, UNK = 2):
    maxlen = max([len(i) for i in corpus])
    X = np.zeros((len(corpus), maxlen))
    for i in range(len(corpus)):
        for no, k in enumerate(corpus[i][:maxlen]):
            X[i, no] = dic.get(k, UNK)
    return X


def generate_char_seq(batch, dic, UNK = 2):
    maxlen_c = max([len(k) for k in batch])
    x = [[len(i) for i in k] for k in batch]
    maxlen = max([j for i in x for j in i])
    temp = np.zeros((len(batch), maxlen_c, maxlen), dtype = np.int32)
    for i in range(len(batch)):
        for k in range(len(batch[i])):
            for no, c in enumerate(batch[i][k][::-1]):
                temp[i, k, -1 - no] = dic.get(c, UNK)
    return temp


def build_dataset(words, n_words, included_prefix = True):
    count = (
        [['GO', 0], ['PAD', 1], ['EOS', 2], ['UNK', 3]]
        if included_prefix
        else []
    )
    count.extend(collections.Counter(words).most_common(n_words))
    dictionary = dict()
    for word, _ in count:
        dictionary[word] = len(dictionary)
    data = list()
    for word in words:
        index = dictionary.get(word, 3)
        data.append(index)
    reversed_dictionary = dict(zip(dictionary.values(), dictionary.keys()))
    return data, count, dictionary, reversed_dictionary



import collections
from unidecode import unidecode
import re
import numpy as np
import tensorflow as tf
from sklearn.utils import shuffle
from tqdm import tqdm
import itertools
from ..cluster import ngrams as ngrams_function


def generator(word, ngrams = (2, 3)):
    return [''.join(i) for n in ngrams for i in ngrams_function(word, n)]


def build_dict(word_counter, vocab_size = 50000):
    count = [['PAD', 0], ['UNK', 1], ['START', 2], ['END', 3]]
    count.extend(word_counter.most_common(vocab_size))
    dictionary = dict()
    for word, _ in count:
        dictionary[word] = len(dictionary)
    return dictionary, {word: idx for idx, word in dictionary.items()}


def doc2num(word_list, dictionary, ngrams = (2, 3)):
    word_array = []
    for word in word_list:
        words = generator(word, ngrams = ngrams)
        word_array.append([dictionary.get(word, 1) for word in words])
    return word_array


def build_word_array(sentences, vocab_size, ngrams = (2, 3)):
    word_counter, word_list, num_lines, num_words = counter_words(
        sentences, ngrams = ngrams
    )
    dictionary, rev_dictionary = build_dict(word_counter, vocab_size)
    word_array = doc2num(word_list, dictionary, ngrams = ngrams)
    return word_array, dictionary, rev_dictionary, num_lines, num_words


def build_training_set(word_array):
    num_words = len(word_array)
    maxlen = max([len(i) for i in word_array])
    x = np.zeros((num_words - 4, maxlen, 4), dtype = np.int32)
    y = np.zeros((num_words - 4, maxlen), dtype = np.int32)
    shift = [-2, -1, 1, 2]
    for idx in range(2, num_words - 2):
        y[idx - 2, : len(word_array[idx])] = word_array[idx]
        for no, s in enumerate(shift):
            x[idx - 2, : len(word_array[idx + s]), no] = word_array[idx + s]
    return x, y


def counter_words(sentences, ngrams = (2, 3)):
    word_counter = collections.Counter()
    word_list = []
    num_lines, num_words = (0, 0)
    for i in sentences:
        words = re.sub('[^\'"A-Za-z\-<> ]+', ' ', unidecode(i))
        word_list.append(words)
        words = generator(words, ngrams = ngrams)
        word_counter.update(words)
        num_lines += 1
        num_words += len(words)
    return word_counter, word_list, num_lines, num_words


class Model:
    def __init__(self, graph_params):
        g_params = graph_params
        _graph = tf.Graph()
        with _graph.as_default():
            if g_params['optimizer'] != 'adam':
                config = tf.ConfigProto(device_count = {'GPU': 0})
                self.sess = tf.InteractiveSession(config = config)
            else:
                self.sess = tf.InteractiveSession()
            self.X = tf.placeholder(tf.int64, shape = [None, None, 4])
            self.Y = tf.placeholder(tf.int64, shape = [None, None])
            w_m2, w_m1, w_p1, w_p2 = tf.unstack(self.X, axis = 2)
            self.embed_weights = tf.Variable(
                tf.random_uniform(
                    [g_params['vocab_size'], g_params['embed_size']],
                    -g_params['embed_noise'],
                    g_params['embed_noise'],
                )
            )
            y = tf.argmax(
                tf.nn.embedding_lookup(self.embed_weights, self.Y), axis = -1
            )
            y = tf.argmax(y, axis = -1)
            y = tf.reshape(y, [-1, 1])
            embed_m2 = tf.reduce_mean(
                tf.nn.embedding_lookup(self.embed_weights, w_m2), axis = 1
            )
            embed_m1 = tf.reduce_mean(
                tf.nn.embedding_lookup(self.embed_weights, w_m1), axis = 1
            )
            embed_p1 = tf.reduce_mean(
                tf.nn.embedding_lookup(self.embed_weights, w_p1), axis = 1
            )
            embed_p2 = tf.reduce_mean(
                tf.nn.embedding_lookup(self.embed_weights, w_p2), axis = 1
            )
            embed_stack = tf.concat([embed_m2, embed_m1, embed_p1, embed_p2], 1)
            hid_weights = tf.Variable(
                tf.random_normal(
                    [g_params['embed_size'] * 4, g_params['hid_size']],
                    stddev = g_params['hid_noise']
                    / (g_params['embed_size'] * 4) ** 0.5,
                )
            )
            hid_bias = tf.Variable(tf.zeros([g_params['hid_size']]))
            hid_out = tf.nn.tanh(tf.matmul(embed_stack, hid_weights) + hid_bias)
            self.nce_weights = tf.Variable(
                tf.random_normal(
                    [g_params['vocab_size'], g_params['hid_size']],
                    stddev = 1.0 / g_params['hid_size'] ** 0.5,
                )
            )
            nce_bias = tf.Variable(tf.zeros([g_params['vocab_size']]))
            self.cost = tf.reduce_mean(
                tf.nn.nce_loss(
                    self.nce_weights,
                    nce_bias,
                    inputs = hid_out,
                    labels = y,
                    num_sampled = g_params['neg_samples'],
                    num_classes = g_params['vocab_size'],
                    num_true = 1,
                    remove_accidental_hits = True,
                )
            )
            self.logits = tf.argmax(
                tf.matmul(hid_out, self.nce_weights, transpose_b = True)
                + nce_bias,
                axis = 1,
            )
            if g_params['optimizer'] == 'rmsprop':
                self.optimizer = tf.train.RMSPropOptimizer(
                    g_params['learn_rate']
                ).minimize(self.cost)
            elif g_params['optimizer'] == 'momentum':
                self.optimizer = tf.train.MomentumOptimizer(
                    g_params['learn_rate'], g_params['momentum']
                ).minimize(self.cost)
            elif g_params['optimizer'] == 'adam':
                self.optimizer = tf.train.AdamOptimizer(
                    g_params['learn_rate']
                ).minimize(self.cost)
            elif g_params['optimizer'] == 'adagrad':
                self.optimizer = tf.train.AdagradOptimizer(
                    g_params['learn_rate']
                ).minimize(self.cost)
            else:
                raise Exception(
                    "Optimizer not supported, only supports ['gradientdescent', 'rmsprop', 'momentum', 'adagrad', 'adam']"
                )
            self.sess.run(tf.global_variables_initializer())

    def train(self, X, Y, X_val, Y_val, epoch, batch_size):
        for i in range(epoch):
            X, Y = shuffle(X, Y)
            pbar = tqdm(
                range(0, len(X), batch_size), desc = 'train minibatch loop'
            )
            for batch in pbar:
                feed_dict = {
                    self.X: X[batch : min(batch + batch_size, len(X))],
                    self.Y: Y[batch : min(batch + batch_size, len(X))],
                }
                _, loss = self.sess.run(
                    [self.optimizer, self.cost], feed_dict = feed_dict
                )
                pbar.set_postfix(cost = loss)

            pbar = tqdm(
                range(0, len(X_val), batch_size), desc = 'test minibatch loop'
            )
            for batch in pbar:
                feed_dict = {
                    self.X: X_val[batch : min(batch + batch_size, len(X_val))],
                    self.Y: Y_val[batch : min(batch + batch_size, len(X_val))],
                }
                loss = self.sess.run(self.cost, feed_dict = feed_dict)
                pbar.set_postfix(cost = loss)
        return self.embed_weights.eval(), self.nce_weights.eval()



import setuptools


__packagename__ = 'malaya'

setuptools.setup(
    name = __packagename__,
    packages = setuptools.find_packages(),
    version = '1.6',
    python_requires = '==3.6.*',
    description = 'Natural-Language-Toolkit for bahasa Malaysia, powered by Deep Learning.',
    author = 'huseinzol05',
    author_email = 'husein.zol05@gmail.com',
    url = 'https://github.com/huseinzol05/Malaya',
    download_url = 'https://github.com/huseinzol05/Malaya/archive/master.zip',
    keywords = ['nlp', 'bm'],
    install_requires = [
        'xgboost==0.80',
        'sklearn',
        'sklearn_crfsuite',
        'scikit-learn==0.19.1',
        'requests',
        'fuzzywuzzy',
        'tqdm',
        'unidecode',
        'tensorflow',
        'numpy',
        'scipy',
        'python-levenshtein',
        'pandas',
        'PySastrawi',
        'toolz',
        'PuLP-py3',
    ],
    license = 'MIT',
    classifiers = [
        'Programming Language :: Python :: 3.6',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Topic :: Text Processing',
    ],
    long_description = open('readme-pypi.rst').read(),
)



import sys
import warnings

if not sys.warnoptions:
    warnings.simplefilter('ignore')

import re
import json
import tensorflow as tf
from unidecode import unidecode
from tensorflow.contrib.seq2seq.python.ops import beam_search_ops
from Sastrawi.Stemmer.StemmerFactory import StemmerFactory
from .texts._tatabahasa import permulaan, hujung
from ._utils._utils import load_graph, check_file, check_available
from .texts._text_functions import (
    pad_sentence_batch,
    stemmer_str_idx,
    classification_textcleaning,
)
from ._utils._paths import PATH_STEM, S3_PATH_STEM
from . import home

factory = None
sastrawi_stemmer = None
PAD = 0
GO = 1
EOS = 2
UNK = 3


def _load_sastrawi():
    global factory, sastrawi_stemmer
    factory = StemmerFactory()
    sastrawi_stemmer = factory.create_stemmer()


def _classification_textcleaning_stemmer_attention(string):
    string = re.sub(
        'http\S+|www.\S+',
        '',
        ' '.join(
            [i for i in string.split() if i.find('#') < 0 and i.find('@') < 0]
        ),
    )
    string = unidecode(string).replace('.', ' . ').replace(',', ' , ')
    string = re.sub('[^A-Za-z ]+', ' ', string)
    string = re.sub(r'[ ]+', ' ', string).strip()
    string = ' '.join(
        [i for i in re.findall('[\\w\']+|[;:\-\(\)&.,!?"]', string) if len(i)]
    )
    string = string.lower().split()
    string = [(naive(word), word) for word in string]
    return (
        ' '.join([word[0] for word in string if len(word[0]) > 1]),
        ' '.join([word[1] for word in string if len(word[0]) > 1]),
    )


def _classification_textcleaning_stemmer(string):
    string = re.sub(
        'http\S+|www.\S+',
        '',
        ' '.join(
            [i for i in string.split() if i.find('#') < 0 and i.find('@') < 0]
        ),
    )
    string = unidecode(string).replace('.', ' . ').replace(',', ' , ')
    string = re.sub('[^A-Za-z ]+', ' ', string)
    string = re.sub(r'[ ]+', ' ', string).strip()
    string = ' '.join(
        [i for i in re.findall('[\\w\']+|[;:\-\(\)&.,!?"]', string) if len(i)]
    )
    string = string.lower().split()
    string = [(naive(word), word) for word in string]
    return ' '.join([word[0] for word in string if len(word[0]) > 1])


class _DEEP_STEMMER:
    def __init__(self, x, logits, sess, dicts):
        self._sess = sess
        self._x = x
        self._logits = logits
        self._dicts = dicts
        self._dicts['rev_dictionary_to'] = {
            int(k): v for k, v in self._dicts['rev_dictionary_to'].items()
        }

    def stem(self, string):
        """
        Stem a string.

        Parameters
        ----------
        string : str

        Returns
        -------
        string: stemmed string
        """
        assert isinstance(string, str), 'input must be a string'
        token_strings = classification_textcleaning(string, True).split()
        idx = stemmer_str_idx(token_strings, self._dicts['dictionary_from'])
        predicted = self._sess.run(
            self._logits, feed_dict = {self._x: pad_sentence_batch(idx, PAD)[0]}
        )
        results = []
        for word in predicted:
            results.append(
                ''.join(
                    [
                        self._dicts['rev_dictionary_to'][c]
                        for c in word
                        if c not in [GO, PAD, EOS, UNK]
                    ]
                )
            )
        return ' '.join(results)


def naive(word):
    """
    Stem a string using startswith and endswith.

    Parameters
    ----------
    string : str

    Returns
    -------
    string: stemmed string
    """
    assert isinstance(word, str), 'input must be a string'
    hujung_result = [e for e in hujung if word.endswith(e)]
    if len(hujung_result):
        hujung_result = max(hujung_result, key = len)
        if len(hujung_result):
            word = hujung_result[: -len(hujung_result)]
    permulaan_result = [e for e in permulaan if word.startswith(e)]
    if len(permulaan_result):
        permulaan_result = max(permulaan_result, key = len)
        if len(permulaan_result):
            word = word[len(permulaan_result) :]
    return word


def available_deep_model():
    """
    List available deep learning stemming models.
    """
    return ['lstm', 'bahdanau', 'luong']


def sastrawi(string):
    """
    Stem a string using Sastrawi.

    Parameters
    ----------
    string : str

    Returns
    -------
    string: stemmed string.
    """
    if sastrawi_stemmer is None:
        _load_sastrawi()
    assert isinstance(string, str), 'input must be a string'
    return sastrawi_stemmer.stem(string)


def deep_model(model = 'bahdanau', validate = True):
    """
    Load seq2seq stemmer deep learning model.

    Returns
    -------
    DEEP_STEMMER: malaya.stemmer._DEEP_STEMMER class
    """
    if validate:
        check_file(PATH_STEM[model], S3_PATH_STEM[model])
    else:
        if not check_available(PATH_STEM[model]):
            raise Exception(
                'stem/%s is not available, please `validate = True`' % (model)
            )
    try:
        with open(PATH_STEM[model]['setting'], 'r') as fopen:
            dic_stemmer = json.load(fopen)
        g = load_graph(PATH_STEM[model]['model'])
    except:
        raise Exception(
            "model corrupted due to some reasons, please run malaya.clear_cache('stem/%s') and try again"
            % (model)
        )
    return _DEEP_STEMMER(
        g.get_tensor_by_name('import/Placeholder:0'),
        g.get_tensor_by_name('import/logits:0'),
        tf.InteractiveSession(graph = g),
        dic_stemmer,
    )



import sys
import warnings

if not sys.warnoptions:
    warnings.simplefilter('ignore')

import pickle
import os
import numpy as np
from fuzzywuzzy import fuzz
from sklearn.manifold import TSNE
from scipy.spatial.distance import cdist
from sklearn.neighbors import NearestNeighbors
from . import home
from ._utils._utils import download_file
from ._models import _fasttext
from .texts._text_functions import simple_textcleaning

_generator = _fasttext.generator
_doc2num = _fasttext.doc2num


def load_wiki():
    """
    Return malaya pretrained wikipedia fast-text size 1024

    Returns
    -------
    dictionary: dictionary of dictionary, reverse dictionary and vectors
    """
    if not os.path.isfile('%s/fasttext-wiki/word2vec.p' % (home)):
        print('downloading fasttext-wiki embedded')
        download_file(
            'v16/fasttext/fasttext-wiki-1024.p',
            '%s/fasttext-wiki/word2vec.p' % (home),
        )
    with open('%s/fasttext-wiki/word2vec.p' % (home), 'rb') as fopen:
        return pickle.load(fopen), (2, 3)


def train(
    corpus,
    vocab_size = None,
    batch_size = 32,
    embedding_size = 512,
    hidden_size = 512,
    negative_samples_ratio = 0.5,
    learning_rate = 0.01,
    embedding_noise = 0.1,
    hidden_noise = 0.3,
    momentum = 0.9,
    epoch = 10,
    optimizer = 'momentum',
    ngrams = (2, 3),
    text_cleaning = simple_textcleaning,
):
    """
    Train a fast-text for custom corpus

    Parameters
    ----------
    corpus: list
        list of strings
    batch_size: int, (default=32)
        batch size for every feed, batch size must <= size of corpus
    embedding_size: int, (default=256)
        vector size representation for a word
    hidden_size: int, (default=256)
        vector size representation for hidden layer
    negative_samples_ratio: float, (default=0.5)
        negative samples ratio proportional to batch_size
    learning_rate: float, (default=0.01)
        learning rate for word2vec
    momentum: float, (default=0.9)
        momentum rate for optimizer=momentum
    epoch: int, (default=5)
        iteration numbers
    optimizer: str, (default='momentum')
        optimizer supported, ['gradientdescent', 'rmsprop', 'momentum', 'adagrad', 'adam']
    ngrams: tuple, (default=(2,3))
        size of ngrams during training session
    text_cleaning: function, (default=simple_textcleaning)
        function to clean the corpus

    Returns
    -------
    dictionary: dictionary of dictionary, reverse dictionary and vectors
    """
    assert isinstance(corpus, str) or isinstance(
        corpus, list
    ), 'corpus must be a string or a list of string'
    assert vocab_size is None or isinstance(
        vocab_size, int
    ), 'vocab_size must be a None or an integer'
    assert isinstance(batch_size, int), 'batch_size must be an integer'
    assert isinstance(embedding_size, int), 'embedding_size must be an integer'
    assert isinstance(hidden_size, int), 'hidden_size must be an integer'
    assert isinstance(epoch, int), 'epoch must be an integer'
    assert isinstance(
        negative_samples_ratio, float
    ), 'negative_samples_ratio must be a float'
    assert isinstance(momentum, float), 'momentum must be a float'
    assert isinstance(embedding_noise, float), 'embedding_noise must be a float'
    assert isinstance(hidden_noise, float), 'hidden_noise must be a float'
    assert isinstance(learning_rate, float) or isinstance(
        learning_rate, int
    ), 'learning_rate must be a float or an integer'
    assert isinstance(optimizer, str), 'optimizer must be a string'
    assert isinstance(ngrams, tuple), 'ngrams must be a tuple'
    assert batch_size > 0, 'batch_size must bigger than 0'
    assert epoch > 0, 'epoch must bigger than 0'
    assert embedding_size > 0, 'embedding_size must bigger than 0'
    assert hidden_size > 0, 'hidden_size must bigger than 0'
    assert (
        negative_samples_ratio > 0 and negative_samples_ratio <= 1
    ), 'negative_samples_ratio must bigger than 0 and less than or equal 1'
    assert (
        embedding_noise > 0 and embedding_noise <= 1
    ), 'embedding_noise must bigger than 0 and less than or equal 1'
    assert (
        hidden_noise > 0 and hidden_noise <= 1
    ), 'hidden_noise must bigger than 0 and less than or equal 1'
    assert isinstance(ngrams[0], int), 'elements in ngrams must be an integer'
    optimizer = optimizer.lower()
    if optimizer not in [
        'gradientdescent',
        'adam',
        'adagrad',
        'momentum',
        'rmsprop',
    ]:
        raise Exception(
            "Optimizer not supported, only supports ['gradientdescent', 'rmsprop', 'momentum', 'adagrad', 'adam']"
        )
    from sklearn.model_selection import train_test_split

    if isinstance(corpus, list):
        corpus = ' '.join(corpus)
    if text_cleaning:
        corpus = text_cleaning(corpus)
    corpus = ' '.join(corpus.split('\n'))
    corpus = list(filter(None, corpus.split()))
    corpus = ['<%s>' % (s) for s in corpus]
    if vocab_size is None:
        vocab_size = len(set(corpus)) + 5
    word_array, dictionary, rev_dictionary, num_lines, num_words = _fasttext.build_word_array(
        corpus, vocab_size, ngrams = ngrams
    )
    X, Y = _fasttext.build_training_set(word_array)
    graph_params = {
        'batch_size': batch_size,
        'vocab_size': np.max(X) + 1,
        'embed_size': embedding_size,
        'hid_size': hidden_size,
        'neg_samples': int(batch_size * negative_samples_ratio),
        'learn_rate': learning_rate,
        'momentum': momentum,
        'embed_noise': embedding_noise,
        'hid_noise': hidden_noise,
        'epoch': epoch,
        'optimizer': optimizer,
    }
    _, test_X, _, test_Y = train_test_split(X, Y, test_size = 0.1)
    model = _fasttext.Model(graph_params)
    print(
        'model built, vocab size %d, document length %d'
        % (np.max(X) + 1, len(word_array))
    )
    embed_weights, nce_weights = model.train(
        X, Y, test_X, test_Y, graph_params['epoch'], graph_params['batch_size']
    )
    return embed_weights, nce_weights, dictionary, ngrams


class _Calculator:
    def __init__(self, tokens):
        self._tokens = tokens
        self._current = tokens[0]

    def exp(self):
        result = self.term()
        while self._current in ('+', '-'):
            if self._current == '+':
                self.next()
                result += self.term()
            if self._current == '-':
                self.next()
                result -= self.term()
        return result

    def factor(self):
        result = None
        if self._current[0].isdigit() or self._current[-1].isdigit():
            result = np.array([float(i) for i in self._current.split(',')])
            self.next()
        elif self._current is '(':
            self.next()
            result = self.exp()
            self.next()
        return result

    def next(self):
        self._tokens = self._tokens[1:]
        self._current = self._tokens[0] if len(self._tokens) > 0 else None

    def term(self):
        result = self.factor()
        while self._current in ('*', '/'):
            if self._current == '*':
                self.next()
                result *= self.term()
            if self._current == '/':
                self.next()
                result /= self.term()
        return result


class fast_text:
    def __init__(self, embed_matrix, dictionary, ngrams):
        self._embed_matrix = embed_matrix
        self._dictionary = dictionary
        self._reverse_dictionary = {v: k for k, v in dictionary.items()}
        self.words = list(dictionary.keys())
        self.ngrams = ngrams

    def to_vector(self, word_list):
        pools = []
        for word in word_list:
            word = filter(None, word.split())
            pools.append(''.join(['<%s>' % (w) for w in word]))
        word_array = _doc2num(pools, self._dictionary, ngrams = self.ngrams)
        outside_array = []
        for arr in word_array:
            outside_array.append(
                np.array([self._embed_matrix[i] for i in arr]).sum(axis = 0)
            )
        return np.array(outside_array)

    def get_vector_by_name(self, word):
        word = filter(None, word.split())
        word = ''.join(['<%s>' % (w) for w in word])
        word_array = _doc2num([word], self._dictionary, ngrams = self.ngrams)[0]
        return np.array([self._embed_matrix[i] for i in word_array]).sum(
            axis = 0
        )

    def tree_plot(
        self,
        labels,
        notebook_mode = False,
        figsize = (7, 7),
        annotate = True,
        figname = 'fig.png',
    ):
        """
        plot a tree plot based on output from calculator / n_closest / analogy

        Parameters
        ----------
        labels : list
            output from calculator / n_closest / analogy
        notebook_mode : bool
            if True, it will render plt.show, else plt.savefig
        figsize : tuple, (default=(7, 7))
            figure size for plot
        figname : str, (default='fig.png')

        Returns
        -------
        list_dictionaries: list of results
        """
        try:
            import matplotlib.pyplot as plt
            import seaborn as sns

            sns.set()
        except:
            raise Exception(
                'matplotlib and seaborn not installed. Please install it and try again.'
            )
        idx = [e[0] if isinstance(e, list) else e for e in labels]
        embed = self.to_vector(idx)
        embed = embed.dot(embed.T)
        embed = (embed - embed.min()) / (embed.max() - embed.min())
        labelled = []
        for label in labels:
            label = (
                '%s, %.3f' % (label[0], label[1])
                if isinstance(label, list)
                else label
            )
            labelled.append(label)
        plt.figure(figsize = figsize)
        g = sns.clustermap(
            embed,
            cmap = 'Blues',
            xticklabels = labelled,
            yticklabels = labelled,
            annot = annotate,
        )
        if notebook_mode:
            plt.show()
        else:
            plt.savefig(figname, bbox_inches = 'tight')

    def scatter_plot(
        self,
        labels,
        centre = None,
        notebook_mode = False,
        figsize = (7, 7),
        figname = 'fig.png',
        handoff = 5e-5,
    ):
        """
        plot a scatter plot based on output from calculator / n_closest / analogy

        Parameters
        ----------
        labels : list
            output from calculator / n_closest / analogy
        centre : str, (default=None)
            centre label, if a str, it will annotate in a red color
        notebook_mode : bool
            if True, it will render plt.show, else plt.savefig
        figsize : tuple, (default=(7, 7))
            figure size for plot
        figname : str, (default='fig.png')

        Returns
        -------
        list_dictionaries: list of results
        """
        assert isinstance(labels, list), 'input must be a list'
        assert isinstance(
            notebook_mode, bool
        ), 'notebook_mode must be a boolean'
        assert isinstance(figsize, tuple), 'figsize must be a tuple'
        assert isinstance(figname, str), 'figname must be a string'
        try:
            import matplotlib.pyplot as plt
            import seaborn as sns

            sns.set()
        except:
            raise Exception(
                'matplotlib and seaborn not installed. Please install it and try again.'
            )
        idx = [e[0] if isinstance(e, list) else e for e in labels]
        if centre:
            assert isinstance(centre, str), 'centre must be a string'
            idx.append(centre)
        embed_matrix = self.to_vector(idx)
        tsne = TSNE(n_components = 2, random_state = 0).fit_transform(
            embed_matrix
        )
        plt.figure(figsize = figsize)
        plt.scatter(tsne[:, 0], tsne[:, 1])
        for label, x, y in zip(labels, tsne[:, 0], tsne[:, 1]):
            label = (
                '%s, %.3f' % (label[0], label[1])
                if isinstance(label, list)
                else label
            )
            plt.annotate(
                label,
                xy = (x, y),
                xytext = (0, 0),
                textcoords = 'offset points',
            )
        if centre:
            plt.annotate(
                centre,
                xy = (tsne[-1, 0], tsne[-1, 1]),
                xytext = (0, 0),
                textcoords = 'offset points',
                color = 'red',
            )
        plt.xlim(
            tsne[: len(idx), 0].min() + handoff,
            tsne[: len(idx), 0].max() + handoff,
        )
        plt.ylim(
            tsne[: len(idx), 1].min() + handoff,
            tsne[: len(idx), 1].max() + handoff,
        )
        plt.xticks([])
        plt.yticks([])
        if notebook_mode:
            plt.show()
        else:
            plt.savefig(figname, bbox_inches = 'tight')

    def calculator(
        self,
        equation,
        words_pool,
        num_closest = 5,
        metric = 'cosine',
        return_similarity = True,
    ):
        """
        calculator parser for word2vec

        Parameters
        ----------
        equation: str
            Eg, '(mahathir + najib) - rosmah'
        words_pool: list
            Eg, ['makan','najib','minum','mahathir pm']
        num_closest: int, (default=5)
            number of words closest to the result
        metric: str, (default='cosine')
            vector distance algorithm
        return_similarity: bool, (default=True)
            if True, will return between 0-1 represents the distance

        Returns
        -------
        word_list: list of nearest words
        """
        assert isinstance(equation, str), 'input must be a string'
        assert isinstance(num_closest, int), 'num_closest must be an integer'
        assert isinstance(metric, str), 'metric must be a string'
        assert isinstance(
            return_similarity, bool
        ), 'num_closest must be a boolean'
        assert isinstance(
            words_pool[0], str
        ), 'elements of words_pool must be a string'
        assert (
            len(words_pool) >= num_closest
        ), 'length of words_pool must bigger or equal than num_closest'
        tokens, temp = [], ''
        for char in equation:
            if char == ' ':
                continue
            if char not in '()*+-':
                temp += char
            else:
                if len(temp):
                    tokens.append(
                        ','.join(
                            self.to_vector([temp])[0].astype('str').tolist()
                        )
                    )
                    temp = ''
                tokens.append(char)
        if len(temp):
            tokens.append(
                ','.join(self.to_vector([temp])[0].astype('str').tolist())
            )

        embed_matrix = self.to_vector(words_pool)
        if return_similarity:
            nn = NearestNeighbors(num_closest, metric = metric).fit(
                embed_matrix
            )
            distances, idx = nn.kneighbors(
                _Calculator(tokens).exp().reshape((1, -1))
            )
            word_list = []
            for i in range(idx.shape[1]):
                word_list.append([words_pool[idx[0, i]], 1 - distances[0, i]])
            return word_list
        else:
            closest_indices = self.closest_row_indices(
                _Calculator(tokens).exp(), num_closest + 1, metric, embed_matrix
            )
            word_list = []
            for i in closest_indices:
                word_list.append(words_pool[i])
            return word_list

    def n_closest(
        self,
        word,
        words_pool,
        num_closest = 5,
        metric = 'cosine',
        return_similarity = True,
    ):
        """
        find nearest words based on a word

        Parameters
        ----------
        word: str
            Eg, 'najib'
        words_pool: list
            Eg, ['makan','najib','minum','mahathir pm']
        num_closest: int, (default=5)
            number of words closest to the result
        metric: str, (default='cosine')
            vector distance algorithm
        return_similarity: bool, (default=True)
            if True, will return between 0-1 represents the distance

        Returns
        -------
        word_list: list of nearest words
        """
        assert isinstance(word, str), 'input must be a string'
        assert isinstance(num_closest, int), 'num_closest must be an integer'
        assert isinstance(metric, str), 'metric must be a string'
        assert isinstance(words_pool, list), 'words_pool must be a list'
        assert isinstance(
            words_pool[0], str
        ), 'elements of words_pool must be a string'
        assert (
            len(words_pool) >= num_closest
        ), 'length of words_pool must bigger or equal than num_closest'
        assert isinstance(
            return_similarity, bool
        ), 'num_closest must be a boolean'
        words_pool.append(word)
        embed_matrix = self.to_vector(words_pool)

        if return_similarity:
            nn = NearestNeighbors(num_closest + 1, metric = metric).fit(
                embed_matrix
            )
            distances, idx = nn.kneighbors(embed_matrix[-1].reshape((1, -1)))
            word_list = []
            for i in range(1, idx.shape[1]):
                word_list.append([words_pool[idx[0, i]], 1 - distances[0, i]])
            return word_list
        else:
            wv = embed_matrix[-1].reshape((1, -1))
            closest_indices = self.closest_row_indices(
                wv, num_closest + 1, metric, embed_matrix
            )
            word_list = []
            for i in closest_indices:
                word_list.append(words_pool[i])
            if word in word_list:
                word_list.remove(word)
            return word_list

    def closest_row_indices(self, wv, num, metric, embed_matrix):
        dist_array = np.ravel(
            cdist(embed_matrix, wv.reshape((1, -1)), metric = metric)
        )
        sorted_indices = np.argsort(dist_array)
        return sorted_indices[:num]

    def analogy(self, a, b, c, words_pool, num = 1, metric = 'cosine'):
        """
        analogy calculation, vb - va + vc

        Parameters
        ----------
        a: str
        b: str
        c: str
        words_pool: list
            Eg, ['makan','najib','minum','mahathir pm']
        num: int, (default=1)
        metric: str, (default='cosine')
            vector distance algorithm

        Returns
        -------
        word_list: list of nearest words
        """
        assert isinstance(a, str), 'a must be a string'
        assert isinstance(b, str), 'b must be a string'
        assert isinstance(c, str), 'c must be a string'
        assert isinstance(
            words_pool[0], str
        ), 'elements of words_pool must be a string'
        assert (
            len(words_pool) >= num
        ), 'length of words_pool must bigger or equal than num_closest'
        words_pool.extend([a, b, c])
        embed_matrix = self.to_vector(words_pool)
        va = embed_matrix[-3]
        vb = embed_matrix[-2]
        vc = embed_matrix[-1]
        vd = vb - va + vc
        closest_indices = self.closest_row_indices(
            vd, num, metric, embed_matrix
        )
        d_word_list = []
        for i in closest_indices:
            d_word_list.append(words_pool[i])
        return d_word_list

    def project_2d(self, words_pool):
        """
        project fast-text into 2d dimension

        Parameters
        ----------
        words_pool: list
            Eg, ['makan','najib','minum','mahathir pm']

        Returns
        -------
        tsne decomposition: numpy
        """
        tsne = TSNE(n_components = 2)
        embed_2d = tsne.fit_transform(embed_matrix = self.to_vector(words_pool))
        return embed_2d, words_pool



import sys
import warnings

if not sys.warnoptions:
    warnings.simplefilter('ignore')

from ._utils import _softmax_class
from ._utils._paths import PATH_EMOTION, S3_PATH_EMOTION

_emotion_label = ['anger', 'fear', 'joy', 'love', 'sadness', 'surprise']


def available_sparse_deep_model():
    """
    List available sparse deep learning emotion analysis models.
    """
    return ['fast-text-char']


def available_deep_model():
    """
    List available deep learning emotion analysis models.
    """
    return [
        'fast-text',
        'hierarchical',
        'bahdanau',
        'luong',
        'bidirectional',
        'bert',
        'entity-network',
    ]


def sparse_deep_model(model = 'fast-text-char', validate = True):
    """
    Load deep learning emotion analysis model.

    Parameters
    ----------
    model : str, optional (default='luong')
        Model architecture supported. Allowed values:

        * ``'fast-text-char'`` - Fast-text architecture for character based n-grams, embedded and logits layers only
    validate: bool, optional (default=True)
        if True, malaya will check model availability and download if not available.

    Returns
    -------
    SPARSE_SOFTMAX: malaya._models._tensorflow_model.SPARSE_SOFTMAX class
    """
    return _softmax_class.sparse_deep_model(
        PATH_EMOTION,
        S3_PATH_EMOTION,
        'emotion',
        _emotion_label,
        len(_emotion_label),
        model = model,
        validate = validate,
    )


def deep_model(model = 'luong', validate = True):
    """
    Load deep learning emotion analysis model.

    Parameters
    ----------
    model : str, optional (default='luong')
        Model architecture supported. Allowed values:

        * ``'fast-text'`` - Fast-text architecture, embedded and logits layers only
        * ``'hierarchical'`` - LSTM with hierarchical attention architecture
        * ``'bahdanau'`` - LSTM with bahdanau attention architecture
        * ``'bidirectional'`` - LSTM with Bidirectional RNN architecture
        * ``'luong'`` - LSTM with luong attention architecture
        * ``'bert'`` - Deep Bidirectional transformers architecture
        * ``'entity-network'`` - Recurrent Entity-Network architecture
    validate: bool, optional (default=True)
        if True, malaya will check model availability and download if not available.

    Returns
    -------
    SOFTMAX: malaya._models._tensorflow_model.SOFTMAX class
    """
    return _softmax_class.deep_model(
        PATH_EMOTION,
        S3_PATH_EMOTION,
        'emotion',
        _emotion_label,
        model = model,
        validate = validate,
    )


def multinomial(validate = True):
    """
    Load multinomial emotion model.

    Parameters
    ----------
    validate: bool, optional (default=True)
        if True, malaya will check model availability and download if not available.

    Returns
    -------
    USER_BAYES : malaya._models._sklearn_model.USER_BAYES class
    """
    return _softmax_class.multinomial(
        PATH_EMOTION,
        S3_PATH_EMOTION,
        'emotion',
        _emotion_label,
        validate = validate,
    )


def xgb(validate = True):
    """
    Load XGB emotion model.

    Parameters
    ----------
    validate: bool, optional (default=True)
        if True, malaya will check model availability and download if not available.

    Returns
    -------
    USER_XGB : malaya._models._sklearn_model.USER_XGB class
    """
    return _softmax_class.xgb(
        PATH_EMOTION,
        S3_PATH_EMOTION,
        'emotion',
        _emotion_label,
        validate = validate,
    )



from tqdm import tqdm
import tensorflow as tf
import requests
import os
from pathlib import Path
from .. import _delete_folder


def download_file(url, filename):
    r = requests.get(
        'http://s3-ap-southeast-1.amazonaws.com/huseinhouse-storage/' + url,
        stream = True,
    )
    total_size = int(r.headers['content-length'])
    os.makedirs(os.path.dirname(filename), exist_ok = True)
    with open(filename, 'wb') as f:
        for data in tqdm(
            iterable = r.iter_content(chunk_size = 1048576),
            total = total_size / 1048576,
            unit = 'MB',
            unit_scale = True,
        ):
            f.write(data)


def load_graph(frozen_graph_filename):
    with tf.gfile.GFile(frozen_graph_filename, 'rb') as f:
        graph_def = tf.GraphDef()
        graph_def.ParseFromString(f.read())
    with tf.Graph().as_default() as graph:
        tf.import_graph_def(graph_def)
    return graph


def check_available(file):
    for key, item in file.items():
        if 'version' in key:
            continue
        if not os.path.isfile(item):
            return False
    return True


def check_file(file, s3_file):
    base_location = os.path.dirname(file['model'])
    version = base_location + '/version'
    download = False
    if os.path.isfile(version):
        with open(version) as fopen:
            if not file['version'] in fopen.read():
                print('Found old version of %s, deleting..' % (base_location))
                _delete_folder(base_location)
                print('Done.')
                download = True
            else:
                for key, item in file.items():
                    if not os.path.isfile(item):
                        download = True
                        break
    else:
        download = True

    if download:
        for key, item in file.items():
            if 'version' in key:
                continue
            if not os.path.isfile(item):
                print('downloading frozen %s %s' % (base_location, key))
                download_file(s3_file[key], item)
        with open(version, 'w') as fopen:
            fopen.write(file['version'])


class DisplayablePath(object):
    display_filename_prefix_middle = '├──'
    display_filename_prefix_last = '└──'
    display_parent_prefix_middle = '    '
    display_parent_prefix_last = '│   '

    def __init__(self, path, parent_path, is_last):
        self.path = Path(str(path))
        self.parent = parent_path
        self.is_last = is_last
        if self.parent:
            self.depth = self.parent.depth + 1
        else:
            self.depth = 0

    @property
    def displayname(self):
        if self.path.is_dir():
            return self.path.name + '/'
        return self.path.name

    @classmethod
    def make_tree(cls, root, parent = None, is_last = False, criteria = None):
        root = Path(str(root))
        criteria = criteria or cls._default_criteria
        displayable_root = cls(root, parent, is_last)
        yield displayable_root

        children = sorted(
            list(path for path in root.iterdir() if criteria(path)),
            key = lambda s: str(s).lower(),
        )
        count = 1
        for path in children:
            is_last = count == len(children)
            if path.is_dir():
                yield from cls.make_tree(
                    path,
                    parent = displayable_root,
                    is_last = is_last,
                    criteria = criteria,
                )
            else:
                yield cls(path, displayable_root, is_last)
            count += 1

    @classmethod
    def _default_criteria(cls, path):
        return True

    @property
    def displayname(self):
        if self.path.is_dir():
            return self.path.name + '/'
        return self.path.name

    def displayable(self):
        if self.parent is None:
            return self.displayname

        _filename_prefix = (
            self.display_filename_prefix_last
            if self.is_last
            else self.display_filename_prefix_middle
        )

        parts = ['{!s} {!s}'.format(_filename_prefix, self.displayname)]

        parent = self.parent
        while parent and parent.parent is not None:
            parts.append(
                self.display_parent_prefix_middle
                if parent.is_last
                else self.display_parent_prefix_last
            )
            parent = parent.parent

        return ''.join(reversed(parts))



import collections
import re
import numpy as np
import tensorflow as tf
from sklearn.utils import shuffle
from tqdm import tqdm


def counter_words(sentences):
    word_counter = collections.Counter()
    word_list = []
    num_lines, num_words = (0, 0)
    for i in sentences:
        words = re.findall('[\\w\']+|[;:\-\(\)&.,!?"]', i)
        word_counter.update(words)
        word_list.extend(words)
        num_lines += 1
        num_words += len(words)
    return word_counter, word_list, num_lines, num_words


def build_dict(word_counter, vocab_size = 50000):
    count = [['PAD', 0], ['UNK', 1], ['START', 2], ['END', 3]]
    count.extend(word_counter.most_common(vocab_size))
    dictionary = dict()
    for word, _ in count:
        dictionary[word] = len(dictionary)
    return dictionary, {word: idx for idx, word in dictionary.items()}


def doc2num(word_list, dictionary):
    word_array = []
    for word in word_list:
        word_array.append(dictionary.get(word, 1))
    return np.array(word_array, dtype = np.int32)


def build_word_array(sentences, vocab_size):
    word_counter, word_list, num_lines, num_words = counter_words(sentences)
    dictionary, rev_dictionary = build_dict(word_counter, vocab_size)
    word_array = doc2num(word_list, dictionary)
    return word_array, dictionary, rev_dictionary, num_lines, num_words


def build_training_set(word_array):
    num_words = len(word_array)
    x = np.zeros((num_words - 4, 4), dtype = np.int32)
    y = np.zeros((num_words - 4, 1), dtype = np.int32)
    shift = np.array([-2, -1, 1, 2], dtype = np.int32)
    for idx in range(2, num_words - 2):
        y[idx - 2, 0] = word_array[idx]
        x[idx - 2, :] = word_array[idx + shift]
    return x, y


class Model:
    def __init__(self, graph_params):
        g_params = graph_params
        _graph = tf.Graph()
        with _graph.as_default():
            if g_params['optimizer'] != 'adam':
                config = tf.ConfigProto(device_count = {'GPU': 0})
                self.sess = tf.InteractiveSession(config = config)
            else:
                self.sess = tf.InteractiveSession()
            self.X = tf.placeholder(tf.int64, shape = [None, 4])
            self.Y = tf.placeholder(tf.int64, shape = [None, 1])
            w_m2, w_m1, w_p1, w_p2 = tf.unstack(self.X, axis = 1)
            self.embed_weights = tf.Variable(
                tf.random_uniform(
                    [g_params['vocab_size'], g_params['embed_size']],
                    -g_params['embed_noise'],
                    g_params['embed_noise'],
                )
            )
            embed_m2 = tf.nn.embedding_lookup(self.embed_weights, w_m2)
            embed_m1 = tf.nn.embedding_lookup(self.embed_weights, w_m1)
            embed_p1 = tf.nn.embedding_lookup(self.embed_weights, w_p1)
            embed_p2 = tf.nn.embedding_lookup(self.embed_weights, w_p2)
            embed_stack = tf.concat([embed_m2, embed_m1, embed_p1, embed_p2], 1)
            hid_weights = tf.Variable(
                tf.random_normal(
                    [g_params['embed_size'] * 4, g_params['hid_size']],
                    stddev = g_params['hid_noise']
                    / (g_params['embed_size'] * 4) ** 0.5,
                )
            )
            hid_bias = tf.Variable(tf.zeros([g_params['hid_size']]))
            hid_out = tf.nn.tanh(tf.matmul(embed_stack, hid_weights) + hid_bias)
            self.nce_weights = tf.Variable(
                tf.random_normal(
                    [g_params['vocab_size'], g_params['hid_size']],
                    stddev = 1.0 / g_params['hid_size'] ** 0.5,
                )
            )
            nce_bias = tf.Variable(tf.zeros([g_params['vocab_size']]))
            self.cost = tf.reduce_mean(
                tf.nn.nce_loss(
                    self.nce_weights,
                    nce_bias,
                    inputs = hid_out,
                    labels = self.Y,
                    num_sampled = g_params['neg_samples'],
                    num_classes = g_params['vocab_size'],
                    num_true = 1,
                    remove_accidental_hits = True,
                )
            )
            self.logits = tf.argmax(
                tf.matmul(hid_out, self.nce_weights, transpose_b = True)
                + nce_bias,
                axis = 1,
            )
            if g_params['optimizer'] == 'rmsprop':
                self.optimizer = tf.train.RMSPropOptimizer(
                    g_params['learn_rate']
                ).minimize(self.cost)
            elif g_params['optimizer'] == 'momentum':
                self.optimizer = tf.train.MomentumOptimizer(
                    g_params['learn_rate'], g_params['momentum']
                ).minimize(self.cost)
            elif g_params['optimizer'] == 'adam':
                self.optimizer = tf.train.AdamOptimizer(
                    g_params['learn_rate']
                ).minimize(self.cost)
            elif g_params['optimizer'] == 'adagrad':
                self.optimizer = tf.train.AdagradOptimizer(
                    g_params['learn_rate']
                ).minimize(self.cost)
            else:
                raise Exception(
                    "Optimizer not supported, only supports ['gradientdescent', 'rmsprop', 'momentum', 'adagrad', 'adam']"
                )
            self.sess.run(tf.global_variables_initializer())

    def train(self, X, Y, X_val, Y_val, epoch, batch_size):
        for i in range(epoch):
            X, Y = shuffle(X, Y)
            pbar = tqdm(
                range(0, len(X), batch_size), desc = 'train minibatch loop'
            )
            for batch in pbar:
                feed_dict = {
                    self.X: X[batch : min(batch + batch_size, len(X))],
                    self.Y: Y[batch : min(batch + batch_size, len(X))],
                }
                _, loss = self.sess.run(
                    [self.optimizer, self.cost], feed_dict = feed_dict
                )
                pbar.set_postfix(cost = loss)

            pbar = tqdm(
                range(0, len(X_val), batch_size), desc = 'test minibatch loop'
            )
            for batch in pbar:
                feed_dict = {
                    self.X: X_val[batch : min(batch + batch_size, len(X_val))],
                    self.Y: Y_val[batch : min(batch + batch_size, len(X_val))],
                }
                loss = self.sess.run(self.cost, feed_dict = feed_dict)
                pbar.set_postfix(cost = loss)
        return self.embed_weights.eval(), self.nce_weights.eval()
